use bevy::prelude::*;
use bevy_rapier3d::{
    prelude::*,
    rapier::{
        control::KinematicCharacterController,
        prelude::{ColliderType, MassProperties},
    },
};
use clap::ValueEnum;

use crate::{
    AppState,
    debug::{ConsoleLog, DebugEvent, DebugMode},
};

#[derive(States, Default, Debug, Clone, Copy, PartialEq, Eq, Hash, Reflect, ValueEnum)]
pub enum PlayerMode {
    #[default]
    Uninitialized,
    Normal,
    FreeCam,
}

pub fn plugin(app: &mut App) {
    app.register_type::<PlayerSpawnMarker>()
        .add_systems(OnEnter(PlayerMode::Normal), init_normal_mode)
        .add_systems(
            Update,
            (dbg_handle_init_player.run_if(
                in_state(PlayerMode::Uninitialized)
                    .and(in_state(DebugMode::Enabled).and(in_state(AppState::Ingame))),
            ),),
        )
        .init_state::<PlayerMode>();
}

const DEFAULT_PLAYER_SIZE: (f32, f32) = (0.3, 1.8);

fn init_normal_mode(
    mut c: Commands,
    tf: Single<&Transform, With<PlayerSpawnMarker>>,
    mut meshes: ResMut<Assets<Mesh>>,
    mut materials: ResMut<Assets<StandardMaterial>>,
) {
    // c.spawn((Collider::cuboid(20., 1., 20.), RigidBody::Fixed));

    c.spawn((
        tf.clone(),
        Mesh3d(meshes.add(Capsule3d::new(DEFAULT_PLAYER_SIZE.0, DEFAULT_PLAYER_SIZE.1))),
        MeshMaterial3d(materials.add(Color::srgb(0., 1., 0.))),
        Collider::capsule_y(DEFAULT_PLAYER_SIZE.1 / 2., DEFAULT_PLAYER_SIZE.0),
        // Collider::cuboid(1., 1., 1.),
        RigidBody::Dynamic,
        // LockedAxes::ROTATION_LOCKED,
    ));
}

fn dbg_handle_init_player(
    mut dbg_reader: EventReader<DebugEvent>,
    mut state: ResMut<NextState<PlayerMode>>,
    mut logger: ResMut<ConsoleLog>,
) {
    for ev in dbg_reader.read() {
        if let DebugEvent::InitPlayer { mode } = ev {
            match mode {
                PlayerMode::Uninitialized => {
                    logger.err("Can't initialize the Player as Uninitialized.")
                }
                other => {
                    state.set(*other);
                    logger.write(&format!("Player initialized as {mode:?}"));
                }
            };
        }
    }
}

/// Marks where the player spawns
#[derive(Component, Clone, Copy, Debug, Reflect)]
#[reflect(Component)]
struct PlayerSpawnMarker;

#[derive(Component, Clone, Debug)]
struct Player;
